<!doctype html>
<html lang="en">
<head>
  <meta charset="UTF-8" />

  <title>{{ title }}</title>
</head>

<body>

<div id="content">
  <p>Dear Administrators, <br />
      A compute node failed and freezer-dr DID NOT successfully evacuate.
      Please, find the following details about the host: <br />
      Host: {{ host }} <br />
      <p>
      Tenants:
      {% for tenant in tenants %}
          {{ tenant.get('id') }} <br />
      {% endfor %}
      </p>
      <br />
      <p>
          Instances:  <br />
      <table>
          <tr>
              <td>
                  Instance Name
              </td>
              <td>
                  IP
              </td>
          </tr>
          {% for instance in instances %}
          <tr>
              <td> {{ instance.get('name') }} </td>
              <td>
                   {% for key, value in instance.get('addresses').iteritems() %}
                  <b>{{ key }}</b> :
                  {{ value[0].get('addr', 'No IP') }}
                {% endfor %}
              </td>
          </tr>
          {% endfor %}
      </table>
      </p>
      <p>
          Host INFO:
          <table>
          {% for key, value in hypervisor.iteritems() %}
          <tr>
              <td> {{ key }} </td>
              <td> {{ value }} </td>
          </tr>
          {% endfor %}
          </table>
      </p>

      TimeStamp: {{ evacuation_time }} <br />
  </p>
    <br />
    <p>
        Thanks for using <b>Freezer-DR !</b>
    </p>
</div>

</body>
</html>
