<!doctype html>
<html lang="en">
<head>
  <meta charset="UTF-8" />

  <title>{{ title }}</title>
</head>

<body>

<div id="content">
  <p>Dear {{ name }}, <br />
      One of our compute nodes failed due to some technical problem. Your
      instances (listed below) running in tenant {{ tenant }} Evacuated to another compute host <br />
      Instances:  <br />
      <table>
          <tr>
              <td>
                  Instance Name
              </td>
              <td>
                  IP
              </td>
          </tr>
          {% for instance in instances%}
          <tr>
              <td> {{ instance.get('name') }} </td>
              <td>
                   {% for key, value in instance.get('addresses').iteritems() %}
                  <b>{{ key }}</b> :
                  {{ value[0].get('addr', 'No IP') }}
                {% endfor %}
              </td>
          </tr>
          {% endfor %}
      </table>
      TimeStamp: {{ evacuation_time }} <br />
  </p>
    <br />
    <p>
        Thanks for using <b>Freezer-DR !</b>
    </p>
</div>

</body>
</html>